<?php
/**
 * @filename TokenInterface.php
 * @author Jerry Yan <792602257@qq.com>
 * @date 2020/12/17 14:23
 */


namespace JerryYan\DSL\Token;


abstract class TokenInterface
{
    /** @var ?TokenInterface 上一个Token */
    protected $prevToken;
    /** @var ?TokenInterface 下一个Token */
    protected $nextToken=NULL;
    /** @var string 原始数据 */
    protected $_raw = "";
    /** @var array 定义别名 */
    static $alias = [];

    public function __construct(string $original)
    {
        $this->_raw = $original;
    }

    public function setPrevToken(?TokenInterface $token): void
    {
        $this->prevToken = $token;
    }
    public function setNextToken(TokenInterface $token): void
    {
        $this->nextToken = $token;
    }

    /**
     * 获取链表第一个元素
     * @return ?$this
     * @author Jerry Yan <792602257@qq.com>
     * @date 2021/1/22 10:01
     */
    public function getFirstToken(): ?TokenInterface
    {
        if ($this->hasPrevToken()) return $this->getPrevToken()->getFirstToken();
        else return $this;
    }
    public function getLastToken(): ?TokenInterface
    {
        if ($this->hasNextToken()) return $this->getNextToken()->getLastToken();
        else return $this;
    }
    public function hasPrevToken(): bool
    {
        return $this->prevToken !== NULL;
    }
    public function hasNextToken(): bool
    {
        return $this->nextToken !== NULL;
    }
    public function getPrevToken(): ?TokenInterface{
        return $this->prevToken;
    }
    public function getNextToken(): ?TokenInterface{
        return $this->nextToken;
    }
    public function getPrevTokenLength(): int
    {
        if ($this->hasPrevToken()) return $this->getPrevToken()->getPrevTokenLength() + 1;
        else return 0;
    }
    public function getNextTokenLength(): int
    {
        if ($this->hasNextToken()) return $this->getNextToken()->getNextTokenLength() + 1;
        else return 0;
    }
    public function getLength(): int
    {
        return $this->getPrevTokenLength() + $this->getNextTokenLength() + 1;
    }
    public function count(): int
    {
        return $this->getLength();
    }
}