<?php
/**
 * @filename Tokenizer.php
 * @author Jerry Yan <792602257@qq.com>
 * @date 2020/12/17 15:22
 */


namespace JerryYan\DSL\Tokenizer;


use JerryYan\DSL\Reader\ReaderInterface;
use JerryYan\DSL\Token\Factory\FactoryInterface;
use JerryYan\DSL\Token\TokenInterface;

class Tokenizer extends TokenizerInterface
{
    /** @var FactoryInterface token工厂 */
    protected $tokenFactory;
    public function __construct(FactoryInterface $tokenFactory)
    {
        $this->tokenFactory = $tokenFactory;
    }

    /**
     * @inheritDoc
     */
    function tokenize(ReaderInterface $reader): ?TokenInterface
    {
        $reader->resetCursor();
        /** @var TokenInterface $lastToken */
        $lastToken = NULL;
        do
        {
            $currentTokenName = $reader->getCurrentToken();
            $currentToken = $this->tokenFactory->getTokenByName($currentTokenName);
            $currentToken->setPrevToken($lastToken);
            if ($lastToken !== NULL) $lastToken->setNextToken($currentToken);
            $lastToken = $currentToken;
        }while($reader->moveToNextToken());
        return $lastToken->getFirstToken();
    }
}